@@ -102,8 +102,9 @@ struct AlphaBrush
   inline void ink( PIX* pix, int step, int a ) 
   {
     int ia = ALPHA_MAX - a;
-    int o=-W/2+1;
+    int o=-W/2;
     AlphaBlend( *(pix+o*step), m_r, m_g, m_b, a, ia );
+    o++;
     for ( ; o<=W/2; o++ ) {
       *(pix+o*step) = m_c;
     } 
@@ -562,7 +563,7 @@ void Canvas::drawRect( const Rect& r, int c, bool fill )
 
 
 
-Window::Window( int w, int h, const char* title, const char* winclass )
+Window::Window( int w, int h, const char* title, const char* winclass, bool fullscreen )
 {
   if ( winclass ) {
     char s[80];
@@ -577,7 +578,7 @@ Window::Window( int w, int h, const char* title, const char* winclass )
   SDL_WM_ToggleFullScreen( SURFACE(this) );
   SDL_ShowCursor( SDL_DISABLE );
 #else
-  m_state = SDL_SetVideoMode( w, h, 0, SDL_SWSURFACE);
+  m_state = SDL_SetVideoMode( w, h, 32, SDL_SWSURFACE | ((fullscreen==true)?(SDL_FULLSCREEN):(0)));
 #endif
   if ( SURFACE(this) == NULL ) {
     throw "Unable to set video mode";
@@ -623,7 +624,9 @@ void Window::raise()
   SDL_VERSION( &sys.version );
   SDL_GetWMInfo( &sys );
 
-#ifndef WIN32
+#if !defined(WIN32) && !(defined(__APPLE__) && defined(__MACH__))
+  /* No X11 stuff on Windows and Mac OS X */
+
   // take focus...
   XEvent ev = { 0 };
   ev.xclient.type         = ClientMessage;
@@ -662,7 +665,7 @@ void Window::setSubName( const char *sub )
 
 Image::Image( const char* file, bool alpha )
 {
-  alpha = false;
+  //alpha = false;
   std::string f( "data/" );
   SDL_Surface* img = IMG_Load((f+file).c_str());
   if ( !img ) {
@@ -672,12 +675,12 @@ Image::Image( const char* file, bool alpha )
   if ( img ) {
     printf("loaded image %s\n",(f+file).c_str());
     if ( alpha ) {
+      SDL_SetColorKey( img,
+ 		       SDL_SRCCOLORKEY|SDL_RLEACCEL,
+ 		       img->format->colorkey );
       m_state = SDL_DisplayFormatAlpha( img );
     } else {
       m_state = SDL_DisplayFormat( img );
-//       SDL_SetColorKey( SURFACE(this),
-// 		       SDL_SRCCOLORKEY|SDL_RLEACCEL,
-// 		       makeColour( 0x00ff00ff ) );
     }
     if ( m_state ) {
       SDL_FreeSurface( img );
@@ -695,11 +698,12 @@ Image::Image( const char* file, bool alpha )
 
 int Canvas::writeBMP( const char* filename ) const
 {
+#pragma pack(push,1)
   typedef struct {
-    unsigned short int type;                 /* Magic identifier            */
-    unsigned int size;                       /* File size in bytes          */
+    unsigned short int type;         /* Magic identifier */
+    unsigned int size;               /* File size in bytes */
     unsigned short int reserved1, reserved2;
-    unsigned int offset;                     /* Offset to image data, bytes */
+    unsigned int offset;             /* Offset to image data, bytes */
   } BMPHEADER;
   
   typedef struct {
@@ -713,12 +717,15 @@ int Canvas::writeBMP( const char* filename ) const
     unsigned int ncolours;           /* Number of colours         */
     unsigned int importantcolours;   /* Important colours         */
   } BMPINFOHEADER;
+  int check_BMPHEADER[(sizeof(BMPHEADER)==14)-1];
+  int check_BMPINFOHEADER[(sizeof(BMPINFOHEADER)==40)-1];
+#pragma pack(pop)
     
   int w = width();
   int h = height();
-  BMPHEADER     head = { 'B'|('M'<<8), 14+40+w*h*3, 0, 0, 0 };
+  BMPHEADER     head = { 'B'|('M'<<8), 14+40+w*h*3, 0, 0, 14+40 };
   BMPINFOHEADER info = { 40, w, h, 1, 24, 0, w*h*3, 100, 100, 0, 0 };
-  
+
   FILE *f = fopen( filename, "wb" );
   if ( f ) {
     Uint32 bpp;
