@@ -20,10 +20,12 @@
 #include "Config.h"
 #include "Path.h"
 #include "Font.h"
-//#include "Worker.h"
+#include "Worker.h"
+#include "Scene.h"
 
 #include <SDL/SDL.h>
 #include <string>
+#include <algorithm>
 
 extern Rect FULLSCREEN_RECT;
 
@@ -63,10 +65,11 @@ public:
   virtual void onTick( int tick ) {}
 
   virtual void draw( Canvas& screen, const Rect& area )
-  {
+  {    
     if ( m_canvas ) {
       screen.drawImage( m_canvas, m_x, m_y );
     }
+    Overlay::draw(screen,area);
     m_isDirty = false;
   }
 
@@ -226,7 +229,7 @@ class UiOverlay : public OverlayBase
 public:
   UiOverlay(GameControl& game, const Rect& pos)
     : OverlayBase( game, pos.tl.x, pos.tl.y, true )
-    , m_bgColour(0xf0f0a0)
+    , m_bgColour(0xdfdfaf)
     , m_fgColour(0)
     , m_motion(0)
   {
@@ -287,14 +290,17 @@ private:
 };
 
 
-class ListOverlay : public UiOverlay
+#if 0
+class GridOverlay : public UiOverlay
 {
 public:
-  ListOverlay( GameControl& game,
-	       ListProvider* provider,
+  GridOverlay( GameControl& game,
+	       GridProvider* provider,
 	       int x, int y,
-	       int w, int h )
+	       int w, int h,
+	       int columns )
     : UiOverlay(game,Rect(x,y,x+w,y+h)),
+      m_columns( columns ),
       m_provider( provider ),
       m_first( 0 ),
       m_current( 0 ),
@@ -306,17 +312,31 @@ public:
   {
     Canvas *first = m_provider->provideItem( m_first, NULL );
     m_itemh = first->height();
-    m_numvis = m_listarea.height() / (m_itemh + m_vgap) + 2;
+    m_itemw = first->width();
+    m_rows = m_provider->countItems()+m_columns-1)/m_columns;
+    m_horizvis = m_listarea.height() / (m_itemh + m_vgap);
+    m_vertvis = m_listarea.width() / (m_itemw + m_hgap);
+    m_numvis = m_horizvis * m_vertvis;
 
     if ( m_numvis <= m_provider->countItems() ) {
       m_allowscroll = false;
     } else {
       m_allowscroll = true;
-      m_listarea.tl.y += 20;
-      m_listarea.br.y -= 20;
-      m_numvis = m_listarea.height() / (m_itemh + m_vgap);
-      addButton( "Back", Rect(0,0,w,20), &ListOverlay::doBack );
-      addButton( "Forward", Rect(0,y-20,w,20), &ListOverlay::doForward );      
+      if ( m_horizvis < m_columns ) {
+	m_listarea.tl.x += 20;
+	m_listarea.br.x -= 20;
+	addButton( "<<", Rect(0,0,h,20), &GridOverlay::doColLeft );
+	addButton( ">>", Rect(x-20,0,h,20), &GridOverlay::doColRight );
+      }
+      if ( m_vertvis < m_rows ) {
+	m_listarea.tl.y += 20;
+	m_listarea.br.y -= 20;
+	addButton( "^^^^", Rect(0,0,w,20), &GridOverlay::doRowUp );
+	addButton( "vvvv", Rect(0,y-20,w,20), &GridOverlay::doRowDown );
+      }
+      m_horizvis = m_listarea.height() / (m_itemh + m_vgap);
+      m_vertvis = m_listarea.wdith() / (m_itemw + m_hgap);
+      m_numvis = m_horizvis * m_vertvis;
     }
 
     m_items = (Canvas**)calloc( sizeof(Canvas*),  m_numvis);
@@ -328,7 +348,7 @@ public:
     renderItems();
   }
 
-  ~ListOverlay()
+  ~GridOverlay()
   {
     for ( int i=0; i<m_numvis; i++ ) {
       if ( m_items[0] ) {
@@ -340,25 +360,29 @@ public:
   void renderItems()
   {
     m_canvas->clear( m_listarea );
-    for ( int i=0; i < m_numvis; i++ ) {
-      int q = m_first + i;
-      if ( q < m_provider->countItems() ) {
-	m_canvas->drawImage( m_items[q%m_numvis], 
-			    m_listarea.tl.x + m_offset.x,
-			    m_listarea.tl.y + m_offset.y + (m_itemh+m_vgap)*i );
-      }
-      if ( q == m_current ) {
-	m_canvas->drawRect( m_listarea.tl.x-1 + m_offset.x,
-			    m_listarea.tl.y + (m_itemh+m_vgap)*i - 1 + m_offset.y,
-			    m_items[q%m_numvis]->width()+2,
-			    m_items[q%m_numvis]->height()+2,
-			    m_fgColour,
-			    false );
+    for ( int r=0; r < m_vertvis; r++ ) {
+      for ( int c=0; c < m_horizvis; c++ ) {
+	int imgindex = r*m_horizvis + c;
+	int item = m_first + r*m_columns + c;
+	if ( m_items[imgindex] ) {
+	  m_canvas->drawImage( m_items[imgindex], 
+			       m_listarea.tl.x + m_offset.x,
+			       m_listarea.tl.y + m_offset.y + (m_itemh+m_vgap)*i );
+	}
+	if ( item == m_current ) {
+	  m_canvas->drawRect( m_listarea.tl.x-1 + m_offset.x,
+			      m_listarea.tl.y + (m_itemh+m_vgap)*i - 1 + m_offset.y,
+			      m_items[imgindex]->width()+2,
+			      m_items[imgindex]->height()+2,
+			      m_fgColour,
+			      false );
+	}
       }
     }
     dirty();
   }
 
+#if 0
   void onTick( int tick )
   {
     UiOverlay::onTick( tick );
@@ -378,6 +402,7 @@ public:
       renderItems();
     }
   }
+#endif
 
   virtual bool onClick( int x, int y ) 
   {
@@ -391,39 +416,68 @@ public:
     }
   }
 
-  virtual bool onKey( int k )
-  {
-    switch (k) {
-    case SDLK_UP:
-      if ( m_current > 0 ) {
-	m_current--;
-	if ( m_current < m_first ) {
-	  doBack();
-	}
-	renderItems();
-      }
-      return true;
-    case SDLK_DOWN:
-      if ( m_current < m_provider->countItems()-1 ) {
-	m_current++;
-	if ( m_current >= m_first+m_numvis-1 ) {
-	  doForward();
-	}
-	renderItems();
-      }
-      return true;
-    }
-    return false;
-  }
-
   bool onDrag( int dx, int dy )
   {
     m_velocity.x = dx;
     m_velocity.y = dy;
   }
 
-  void doBack()
+  void doRowUp() { pan(0,-5); }
+  void doRowDown() { pan(0,5); }
+  void doColLeft() { pan(-5,0); }
+  void doColRight() { pan(5,0); }
+
+  void pan( int dx, int dy )
   {
+    Vec2 motion(0,0);
+    m_offset += Vec2(dx,dy);
+    while ( m_offset.x < 0 ) {
+      if (m_first%m_columns == 0) {
+	m_offset.x = 0;
+      } else {
+	motion.x--;
+	m_first--;
+	m_offset.x += m_itemw;
+      }
+    }
+    while ( m_offset.x > m_itemw ) {
+      if (m_first%m_columns == m_columns-m_horizvis) {
+	m_offset.x = 0;
+      } else {
+	motion.x++;
+	m_first++;
+	m_offset.x -= m_itemw;
+      }
+    }
+    while ( m_offset.y < 0 ) {
+      if (m_first/m_columns == 0) {
+	m_offset.y = 0;
+      } else {
+	motion.y--;
+	m_first-=m_columns;
+	m_offset.y += m_itemh;
+      }
+    }
+    while ( m_offset.x > m_itemh ) {
+      if (m_first/m_columns == m_rows-m_vertvis) {
+	m_offset.y = 0;
+      } else {
+	motion.y++;
+	m_first+=m_columns;
+	m_offset.y -= m_itemh;
+      }
+    }
+
+    if ( motion.x < 0 ) {
+      // shuffle columsn right.
+    } else if ( motion.x > 0 ) {
+      // shuffle columns left.
+    }
+    if ( motion.y < 0 ) {
+      // shuffle rows down.
+    } else if ( motion.y > 0 ) {
+      // shuffle rows up.
+    }
     if ( m_first > 0 ) {
       m_first--;      
       m_items[m_first%m_numvis] = m_provider->provideItem( m_first,
@@ -431,22 +485,17 @@ public:
     }
   }
 
-  void doForward()
-  {
-    int last = m_first + m_numvis - 1;
-    if ( last < m_provider->countItems() ) {
-      m_first++;
-      last = m_first + m_numvis - 1;
-      m_items[last%m_numvis] = m_provider->provideItem( last, m_items[last%m_numvis] );
-    }
-  }
-
 private:
+  int          m_columns;
+  int          m_rows;
   ListProvider *m_provider;
   int          m_first;
   int          m_current;
+  int          m_horizvis
+  int          m_vertvis;
   int          m_numvis;
   int          m_itemh;
+  int          m_itemw;
   int          m_hgap, m_vgap;
   bool         m_allowscroll;
   Rect         m_listarea;
@@ -454,7 +503,7 @@ private:
   Vec2         m_velocity;
   Canvas**     m_items;
 };
-
+#endif
 
 class MenuOverlay : public UiOverlay
 {
@@ -472,39 +521,106 @@ public:
   
 };
 
+
+class LevelThumbnailer : public Worker
+{
+public:
+  LevelThumbnailer( Levels& levels, int sel )
+    : m_levels(levels),
+      m_selected(sel),
+      m_thumb(0)
+  {}
+  virtual void main()
+  {    
+    fprintf(stderr,"creating thumb\n");
+    Canvas temp( SCREEN_WIDTH, SCREEN_HEIGHT );
+    Scene scene( true );
+    int size = m_levels.load( m_selected, m_buf, sizeof(m_buf) );
+    if ( size && scene.load( m_buf, size ) ) {
+      scene.draw( temp, FULLSCREEN_RECT );
+      m_thumb = temp.scale( ICON_SCALE_FACTOR );
+      fprintf(stderr,"thumb done\n");
+    }
+  }
+  int level() { return m_selected; }
+  Canvas* thumb() { return m_thumb; }
+private:
+  unsigned char m_buf[64*1024];
+  Levels&       m_levels;
+  int           m_selected;
+  Canvas*       m_thumb;
+};
+
+
 class NextLevelOverlay : public UiOverlay
 {
 public:
   NextLevelOverlay( GameControl& game )
-    : UiOverlay(game,Rect(FULLSCREEN_RECT.centroid().x-200,
+    : UiOverlay(game,Rect(FULLSCREEN_RECT.centroid().x-220,
 			  FULLSCREEN_RECT.centroid().y-120,
-			  FULLSCREEN_RECT.centroid().x+200,
-			  FULLSCREEN_RECT.centroid().y+120 ) ),
+			  FULLSCREEN_RECT.centroid().x+220,
+			  FULLSCREEN_RECT.centroid().y+160 ) ),
       m_levelIcon(-2),
-      m_icon(NULL)
+      m_icon(NULL),
+      m_worker(NULL)
   {
-    Font::titleFont()->drawCenter( m_canvas, Vec2(200,32), "BRAVO!", 0 );
+    add(new Label("hello"),10,10);
+    const Font *f = Font::titleFont();
+    f->drawCenter( m_canvas, Vec2(220,30), "BRAVO!", 0 );
+    f = Font::headingFont();
+    f->drawLeft( m_canvas, Vec2(20,50),  "Stats:", 0 );
+    f->drawLeft( m_canvas, Vec2(240,50), "Next:", 0 );
 
-    addHotSpot( Rect(0,    0,100,180), &NextLevelOverlay::doPrevLevel );
-    addHotSpot( Rect(300,  0,400,180), &NextLevelOverlay::doNextLevel );
-
-    addButton( "<<", Rect(3,120,40,160),
+    addButton( "<<", Rect(230,210,280,230),
 	       &NextLevelOverlay::doPrevLevel );
-    addButton( "Action Replay", Rect(0,180,200,240),
+    addButton( ">>", Rect(390,210,420,230),
+	       &NextLevelOverlay::doNextLevel );
+    addButton( "Review", Rect(0,240,200,280),
 	       &NextLevelOverlay::doActionReplay );
-    addButton( "Continue", Rect(200,180,400,240),
+    addButton( "Continue", Rect(200,240,400,280),
 	       &NextLevelOverlay::doContinue );
   }
   ~NextLevelOverlay()
   {
+    delete m_worker;
     delete m_icon;
   }
+  virtual bool handleEvent( SDL_Event& ev )
+  {
+    switch( ev.type ) {      
+    case SDL_USEREVENT:
+      if (ev.user.code==WORKER_DONE && m_worker && m_worker->done()) {
+	genIcon();
+      }
+      return false;
+    default:
+      return UiOverlay::handleEvent(ev);
+    }
+  }
   virtual void onShow()
   {
     UiOverlay::onShow();
     m_game.m_refresh = true; //for fullscreen fade
     m_game.m_fade = true;
     m_selectedLevel = m_game.m_level+1;
+
+    m_canvas->clear(Rect(20,80,220,210));
+    char buf[80];
+    const GameStats& stats = m_game.stats();
+    const Font *f = Font::blurbFont();    
+    f->drawLeft( m_canvas, Vec2(40,90),  "time", 0 );
+    sprintf(buf,"%.02f", (float)(stats.endTime - stats.startTime)/1000.0f );
+    f->drawRight( m_canvas, Vec2(160,90),  buf, 0 );
+    f->drawLeft( m_canvas, Vec2(40,110), "strokes", 0 );
+    sprintf(buf,"%d", stats.strokeCount );
+    f->drawRight( m_canvas, Vec2(160,110),  buf, 0 );
+    f->drawLeft( m_canvas, Vec2(50,130), "undone", 0 );
+    sprintf(buf,"%d", stats.undoCount );
+    f->drawRight( m_canvas, Vec2(160,130),  buf, 0 );
+    f->drawLeft( m_canvas, Vec2(50,150), "paused", 0 );
+    sprintf(buf,"%d", stats.pausedStrokes );
+    f->drawRight( m_canvas, Vec2(160,150),  buf, 0 );
+    genIcon();
   }
   virtual void onHide()
   {
@@ -515,12 +631,18 @@ public:
   virtual void draw( Canvas& screen, const Rect& area )
   {
     UiOverlay::draw( screen, area );
+    std::string caption = m_game.levels().levelName(m_selectedLevel);
+    if (caption[0]=='L') {
+      int s = caption.find("_");
+      if (s>1 && s<5) {
+	caption = caption.substr(s+1);
+      }
+    }
+    std::replace(caption.begin(),caption.end(),'_',' ');
+    Font::blurbFont()->drawCenter( &screen, Vec2(m_x+330,m_y+220), 
+				   caption, 0 );
     if ( genIcon() ) {      
-      Font::blurbFont()->drawCenter( &screen, Vec2(m_x+200,m_y+70), 
-				       m_game.levels().levelName(m_selectedLevel), 0 );
-      screen.drawImage( m_icon, m_x+100, m_y+75 );
-    } else {
-      dirty();
+      screen.drawImage( m_icon, m_x+230, m_y+90 );
     }
   }
   void doPrevLevel()
@@ -546,29 +668,33 @@ public:
 private:
   bool genIcon()
   {
-    if ( m_icon==NULL || m_levelIcon != m_selectedLevel ) {
-
-      //Worker w( NULL, 1, 100, 200 );
-
+    if ( m_icon && m_levelIcon != m_selectedLevel ) {
       delete m_icon;
       m_icon = NULL;
-      if ( m_selectedLevel < m_game.levels().numLevels() ) {
-	Canvas temp( SCREEN_WIDTH, SCREEN_HEIGHT );
-	if ( m_game.renderScene( temp, m_selectedLevel) ) {
-	  m_icon = temp.scale( ICON_SCALE_FACTOR );
-	}
-      } else {
+    }
+    if ( m_icon==NULL ) {
+      if ( m_selectedLevel >= m_game.levels().numLevels() ) {
 	m_icon = new Image("theend.png");
 	m_caption = "no more levels!";
-	m_selectedLevel = m_game.levels().numLevels();
+	m_levelIcon = m_selectedLevel = m_game.levels().numLevels();
+      } else if (m_worker==NULL) {
+	m_worker = new LevelThumbnailer(m_game.levels(),m_selectedLevel);
+	m_worker->start();
+      } else if (m_worker->done()) {
+	m_levelIcon = m_worker->level();
+	m_icon = m_worker->thumb();
+	delete m_worker;
+	m_worker = NULL;
+	dirty();
       }
-      m_levelIcon = m_selectedLevel;
+
     }
-    return m_icon;
+    return m_icon!=NULL;
   }
   int          m_selectedLevel;
   int          m_levelIcon;
   Canvas*      m_icon;
+  LevelThumbnailer *m_worker;
   std::string  m_caption;
 };
 
@@ -655,13 +781,16 @@ Overlay* createNextLevelOverlay( GameControl& game )
   return new NextLevelOverlay( game );
 }
 
-Overlay* createListOverlay( GameControl& game,
-			    ListProvider* provider,
+#if 0
+Overlay* createGridOverlay( GameControl& game,
+			    GridProvider* provider,
 			    int x, int y,
-			    int w, int h )
+			    int w, int h,
+			    int columns )
 {
-  return new ListOverlay( game, provider, x, y, w, h );
+  return new GridOverlay( game, provider, x, y, w, h, columns );
 }
+#endif
 
 Overlay* createMenuOverlay( GameControl& game )
 {
