@@ -17,6 +17,7 @@
 #include "Array.h"
 #include "Config.h"
 #include "Scene.h"
+#include "Accelerometer.h"
 
 #include <sstream>
 #include <fstream>
@@ -71,6 +72,15 @@ void configureScreenTransform( int w, int h )
 }
 
 
+struct Joint
+{
+  Joint( Stroke *j1, Stroke* j2, unsigned char e )
+    : joiner(j1), joinee(j2), end(e) {}
+  Stroke *joiner;
+  Stroke *joinee;
+  unsigned char end; //of joiner
+};
+
 class Stroke
 {
 public:
@@ -157,6 +167,7 @@ public:
     if ( m_rawPath.size() < 2 ) {
       throw "invalid stroke def";
     }
+    fprintf(stderr,"created stroke with %d points\n",m_rawPath.size());
     m_origin = m_rawPath.point(0);
     m_rawPath.translate( -m_origin );
     setAttribute( ATTRIB_DUMMY );
@@ -249,6 +260,40 @@ public:
     transform();
   }
 
+  void determineJoints( Stroke* other, Array<Joint>& joints )
+  {
+    if ( (m_attributes&ATTRIB_CLASSBITS)
+	 != (other->m_attributes&ATTRIB_CLASSBITS)
+	 || hasAttribute(ATTRIB_GROUND)
+	 || hasAttribute(ATTRIB_UNJOINABLE)
+	 || other->hasAttribute(ATTRIB_UNJOINABLE)) {
+      // cannot joint goals or tokens to other things
+      // and no point jointing ground endpts
+      return;
+    } 
+
+    transform();
+    for ( unsigned char end=0; end<2; end++ ) {
+      if ( !m_jointed[end] ) {
+	const Vec2& p = m_xformedPath.endpt(end);
+	if ( other->distanceTo( p ) <= JOINT_TOLERANCE ) {
+	  joints.append( Joint(this,other,end) );
+	}
+      }
+    }
+  }
+
+  void join( b2World* world, Stroke* other, unsigned char end )
+  {
+    if ( !m_jointed[end] ) {
+      b2Vec2 p = m_xformedPath.endpt( end );
+      p *= 1.0f/PIXELS_PER_METREf;
+      JointDef j( m_body, other->m_body, p );
+      world->CreateJoint( &j );
+      m_jointed[end] = true;
+    }
+  }
+
   bool maybeCreateJoint( b2World& world, Stroke* other )
   {
     if ( (m_attributes&ATTRIB_CLASSBITS)
@@ -279,13 +324,27 @@ public:
     return true; ///nothing to do
   }
 
-  void draw( Canvas& canvas )
+  void draw( Canvas& canvas, bool drawJoints=false )
   {
     if ( m_hide < HIDE_STEPS ) {
+      int colour = canvas.makeColour(m_colour);
       bool thick = (canvas.width() > 400);
       transform();
-      canvas.drawPath( m_screenPath, canvas.makeColour(m_colour), thick );
+      canvas.drawPath( m_screenPath, colour, thick );
       m_drawn = true;
+      
+      if ( drawJoints ) {
+	int jointcolour = canvas.makeColour(0xff0000);
+	for ( int e=0; e<2; e++ ) {
+	  if (m_jointed[e]) {
+	    const Vec2& pt = m_screenPath.endpt(e);
+	    //canvas.drawPixel( pt.x, pt.y, jointcolour );
+	    //canvas.drawRect( pt.x-1, pt.y-1, 3, 3, jointcolour );
+	    canvas.drawRect( pt.x-1, pt.y, 3, 1, jointcolour );
+	    canvas.drawRect( pt.x, pt.y-1, 1, 3, jointcolour );
+	  }
+	}
+      }
     }
     m_drawnBbox = m_screenBbox;
   }
@@ -373,6 +432,11 @@ public:
     return m_rawPath.numPoints();
   }
 
+  const Vec2& endpt( unsigned char end ) 
+  {
+    return m_xformedPath.endpt(end);
+  }
+
 private:
   static float32 vec2Angle( b2Vec2 v ) 
   {
@@ -384,6 +448,7 @@ private:
     float32 thresh = SIMPLIFY_THRESHOLDf;
     m_rawPath.simplify( thresh );
     m_shapePath = m_rawPath;
+    //fprintf(stderr,"simplified stroke to %d points\n",m_rawPath.size());
 
     while ( m_shapePath.numPoints() > MULTI_VERTEX_LIMIT ) {
       thresh += SIMPLIFY_THRESHOLDf;
@@ -459,16 +524,18 @@ private:
 Scene::Scene( bool noWorld )
   : m_world( NULL ),
     m_bgImage( NULL ),
-    m_protect( 0 )
+    m_protect( 0 ),
+    m_gravity(0.0f, GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf),
+    m_dynamicGravity(false),
+    m_accelerometer(Os::get()->getAccelerometer())
 {
   if ( !noWorld ) {
     b2AABB worldAABB;
     worldAABB.lowerBound.Set(-100.0f, -100.0f);
     worldAABB.upperBound.Set(100.0f, 100.0f);
     
-    b2Vec2 gravity(0.0f, 10.0f*PIXELS_PER_METREf/GRAVITY_FUDGEf);
     bool doSleep = true;
-    m_world = new b2World(worldAABB, gravity, doSleep);
+    m_world = new b2World(worldAABB, m_gravity, doSleep);
     m_world->SetContactListener( this );
   }
 }
@@ -490,6 +557,7 @@ Stroke* Scene::newStroke( const Path& p, int colour, int attribs ) {
   default: s->setColour( brushColours[colour] ); break;
   }
   m_strokes.append( s );
+  m_recorder.newStroke( p, colour, attribs );
   return s;
 }
 
@@ -498,7 +566,9 @@ bool Scene::deleteStroke( Stroke *s ) {
     int i = m_strokes.indexOf(s);
     if ( i >= m_protect ) {
 	reset(s);
-	m_strokes.erase( m_strokes.indexOf(s) );
+	m_strokes.erase( i );
+	m_deletedStrokes.append( s );
+	m_recorder.deleteStroke( i );
 	return true;
     }
   }
@@ -509,7 +579,11 @@ bool Scene::deleteStroke( Stroke *s ) {
 void Scene::extendStroke( Stroke* s, const Vec2& pt )
 {
   if ( s ) {
-    s->addPoint( pt );
+    int i = m_strokes.indexOf(s);
+    if ( i >= m_protect ) {
+      s->addPoint( pt );
+      m_recorder.extendStroke( i, pt );
+    }
   }
 }
 
@@ -519,11 +593,32 @@ void Scene::moveStroke( Stroke* s, const Vec2& origin )
     int i = m_strokes.indexOf(s);
     if ( i >= m_protect ) {
       s->origin( origin );
+      m_recorder.moveStroke( i, origin );
     }
   }
 }
 	
 
+bool Scene::activateStroke( Stroke *s )
+{
+  activate(s);
+  m_recorder.activateStroke( m_strokes.indexOf(s) );
+}
+
+void Scene::getJointCandidates( Stroke* s, Path& pts )
+{
+  Array<Joint> joints;
+  for ( int j=m_strokes.size()-1; j>=0; j-- ) {      
+    if ( s != m_strokes[j] ) {
+      s->determineJoints( m_strokes[j], joints );
+      m_strokes[j]->determineJoints( s, joints );
+    }
+  }
+  for ( int j=joints.size()-1; j>=0; j-- ) {
+    pts.append( joints[j].joiner->endpt(joints[j].end) );
+  }
+}
+
 bool Scene::activate( Stroke *s )
 {
   if ( s->numPoints() > 1 ) {
@@ -546,31 +641,56 @@ void Scene::activateAll()
 
 void Scene::createJoints( Stroke *s )
 {
+  if ( s->body()==NULL ) {
+    return;
+  }
+  Array<Joint> joints;
   for ( int j=m_strokes.size()-1; j>=0; j-- ) {      
-    if ( s != m_strokes[j] ) {
+    if ( s != m_strokes[j] && m_strokes[j]->body() ) {
 	//printf("try join to %d\n",j);
-	s->maybeCreateJoint( *m_world, m_strokes[j] );
-	m_strokes[j]->maybeCreateJoint( *m_world, s );
+      s->determineJoints( m_strokes[j], joints );
+      m_strokes[j]->determineJoints( s, joints );
+      for ( int i=0; i<joints.size(); i++ ) {
+	joints[i].joiner->join( m_world, joints[i].joinee, joints[i].end );
+      }
+      joints.empty();
     }
   }    
 }
 
-void Scene::step()
+void Scene::step( bool isPaused )
 {
-  m_world->Step( ITERATION_TIMESTEPf, SOLVER_ITERATIONS );
-  // clean up delete strokes
-  for ( int i=0; i< m_strokes.size(); i++ ) {
-    if ( m_strokes[i]->hasAttribute(ATTRIB_DELETED) ) {
+  m_recorder.tick(isPaused);
+  isPaused |= m_player.tick();
+
+  if ( !isPaused ) {
+    if (m_dynamicGravity && m_accelerometer) {
+      float32 gx, gy, gz;
+      if ( m_accelerometer->poll( gx, gy, gz ) ) {
+	//fprintf(stderr,"dynamic grav = %f,%f\n", gx, gy );
+	const float32 factor = GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf;
+	b2Vec2 accel( m_gravity.x + gx*factor, 
+		      m_gravity.y + gy*factor );
+	m_world->SetGravity( accel );
+	//TODO record gravity
+      }
+    }
+
+    m_world->Step( ITERATION_TIMESTEPf, SOLVER_ITERATIONS );
+    // clean up delete strokes
+    for ( int i=0; i< m_strokes.size(); i++ ) {
+      if ( m_strokes[i]->hasAttribute(ATTRIB_DELETED) ) {
 	m_strokes[i]->clearAttribute(ATTRIB_DELETED);
 	m_strokes[i]->hide();
-    }	   
-  }
-  // check for token respawn
-  for ( int i=0; i < m_strokes.size(); i++ ) {
-    if ( m_strokes[i]->hasAttribute( ATTRIB_TOKEN )
+      }	   
+    }
+    // check for token respawn
+    for ( int i=0; i < m_strokes.size(); i++ ) {
+      if ( m_strokes[i]->hasAttribute( ATTRIB_TOKEN )
 	   && !BOUNDS_RECT.intersects( m_strokes[i]->worldBbox() ) ) {
 	reset( m_strokes[i] );
 	activate( m_strokes[i] );	  
+      }
     }
   }
 }
@@ -607,32 +727,25 @@ bool Scene::isCompleted()
 
 Rect Scene::dirtyArea()
 {
-  Rect r(0,0,0,0),temp;
-  int numDirty = 0;
+  Rect r(false);
   for ( int i=0; i<m_strokes.size(); i++ ) {
     if ( m_strokes[i]->isDirty() ) {
-	// acumulate new areas to draw
-	temp = m_strokes[i]->screenBbox();
-	if ( !temp.isEmpty() ) {
-	  if ( numDirty==0 ) {	
-	    r = temp;
-	  } else {
-	    r.expand( m_strokes[i]->screenBbox() );
-	  }
-	  // plus prev areas to erase
-	  r.expand( m_strokes[i]->lastDrawnBbox() );
-	  numDirty++;
-	}
+      // acumulate new areas to draw
+      r.expand( m_strokes[i]->screenBbox() );
+      // plus prev areas to erase
+      r.expand( m_strokes[i]->lastDrawnBbox() );
     }
   }
+  for ( int i=0; i<m_deletedStrokes.size(); i++ ) {
+    // acumulate new areas to draw
+    r.expand( m_strokes[i]->lastDrawnBbox() );
+  }
   if ( !r.isEmpty() ) {
     // expand to allow for thick lines
-    r.tl.x--; r.tl.y--;
-    r.br.x++; r.br.y++;
+    r.grow(1);
   }
   return r;
 }
-
 void Scene::draw( Canvas& canvas, const Rect& area )
 {
   if ( m_bgImage ) {
@@ -651,16 +764,23 @@ void Scene::draw( Canvas& canvas, const Rect& area )
 	m_strokes[i]->draw( canvas );
     }
   }
-  //canvas.drawRect( area, 0xffff0000, false );
+  while ( m_deletedStrokes.size() ) {
+    delete m_deletedStrokes[0];
+    m_deletedStrokes.erase(0);
+  }
 }
 
-void Scene::reset( Stroke* s )
+void Scene::reset( Stroke* s, bool purgeUnprotected )
 {
+  while ( purgeUnprotected && m_strokes.size() > m_protect ) {
+    m_strokes[m_strokes.size()-1]->reset(m_world);
+    m_strokes.erase( m_strokes.size()-1 );
+  }
   for ( int i=0; i<m_strokes.size(); i++ ) {
     if (s==NULL || s==m_strokes[i]) {
 	m_strokes[i]->reset(m_world);
     }
-  }    
+  }
 }
 
 Stroke* Scene::strokeAtPoint( const Vec2 pt, float32 max )
@@ -684,23 +804,44 @@ void Scene::clear()
     delete m_strokes[0];
     m_strokes.erase(0);
   }
+  while ( m_deletedStrokes.size() ) {
+    delete m_deletedStrokes[0];
+    m_deletedStrokes.erase(0);
+  }
   if ( m_world ) {
     //step is required to actually destroy bodies and joints
     m_world->Step( ITERATION_TIMESTEPf, SOLVER_ITERATIONS );
   }
+  m_log.empty();
+}
+
+void Scene::setGravity( const b2Vec2& g )
+{
+  m_gravity = g;
+  m_world->SetGravity( m_gravity );
 }
 
 void Scene::setGravity( const std::string& s )
 {
+  m_dynamicGravity = false;
+  setGravity( b2Vec2(0.0f, GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf) );
+
+  for (int i=0; i<s.find(':'); i++) {
+    switch (s[i]) {
+    case 'd': m_dynamicGravity = true; break;
+    }
+  }
+
+  std::string vector = s.substr(s.find(':')+1);
   float32 x,y;      
-  if ( sscanf( s.c_str(), "%f,%f", &x, &y )==2) {
+  if ( sscanf( vector.c_str(), "%f,%f", &x, &y )==2) {
     if ( m_world ) {
 	b2Vec2 g(x,y);
 	g *= PIXELS_PER_METREf/GRAVITY_FUDGEf;
-	m_world->SetGravity( g );
+	setGravity( g );
     }
   } else {
-    fprintf(stderr,"invalid gravity vector\n");
+    fprintf(stderr,"invalid gravity vector [%s]\n",vector.c_str());
   }
 }
 
@@ -721,7 +862,7 @@ bool Scene::load( std::istream& in )
 {
   clear();
   if ( g_bgImage==NULL ) {
-    g_bgImage = new Image("paper.jpg");
+    g_bgImage = new Image("paper.png");
     g_bgImage->scale( SCREEN_WIDTH, SCREEN_HEIGHT );
   }
   m_bgImage = g_bgImage;
@@ -731,9 +872,24 @@ bool Scene::load( std::istream& in )
     parseLine( line );
   }
   protect();
+  printf("loaded log=%d\n",m_log.size());
   return true;
 }
 
+
+void Scene::start( bool replay )
+{
+  activateAll();
+  if ( replay ) {
+    m_recorder.stop();
+    m_player.start( &m_log, this );
+  } else {
+    m_player.stop();
+    m_recorder.start( &m_log );
+  }
+}
+
+
 bool Scene::parseLine( const std::string& line )
 {
   try {
@@ -742,7 +898,8 @@ bool Scene::parseLine( const std::string& line )
     case 'B': m_bg = line.substr(line.find(':')+1);     return true;
     case 'A': m_author = line.substr(line.find(':')+1); return true;
     case 'S': m_strokes.append( new Stroke(line) );     return true;
-    case 'G': setGravity(line.substr(line.find(':')+1));return true;
+    case 'G': setGravity(line);                         return true;
+    case 'E': m_log.append(line.substr(line.find(':')+1));return true;
     }
   } catch ( const char* e ) {
     printf("Stroke error: %s\n",e);
@@ -755,17 +912,27 @@ void Scene::protect( int n )
   m_protect = (n==-1 ? m_strokes.size() : n );
 }
 
-bool Scene::save( const std::string& file )
+bool Scene::save( const std::string& file, bool saveLog )
 {
   printf("saving to %s\n",file.c_str());
+  if ( saveLog ) {      
+    reset();
+  }
   std::ofstream o( file.c_str(), std::ios::out );
   if ( o.is_open() ) {
     o << "Title: "<<m_title<<std::endl;
     o << "Author: "<<m_author<<std::endl;
     o << "Background: "<<m_bg<<std::endl;
-    for ( int i=0; i<m_strokes.size(); i++ ) {
+    for ( int i=0; i<m_strokes.size() && (!saveLog || i<m_protect); i++ ) {
 	o << m_strokes[i]->asString();
     }
+
+    if ( saveLog ) {      
+      for ( int i=0; i<m_log.size(); i++ ) {
+	o << "E: " << m_log.asString( i ) <<std::endl;
+      }
+    }
+
     o.close();
     return true;
   } else {
